"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/index.ts
var src_exports = {};
__export(src_exports, {
  GLOBAL_EFFECT_UNHANDLED_ERROR$: () => GLOBAL_EFFECT_UNHANDLED_ERROR$,
  OBJECT_COMPARATOR: () => OBJECT_COMPARATOR,
  compute: () => compute,
  createAction: () => createAction,
  createController: () => createController,
  createEffect: () => createEffect,
  createEffectController: () => createEffectController,
  createScope: () => createScope,
  createStore: () => createStore,
  createStoreLoggerExtension: () => createStoreLoggerExtension,
  createStoreUpdates: () => createStoreUpdates,
  declareController: () => declareController,
  declareStateUpdates: () => declareStateUpdates,
  declareStore: () => declareStore,
  declareViewController: () => declareViewController,
  mapQuery: () => mapQuery,
  mergeQueries: () => mergeQueries,
  pipeStateMutations: () => pipeStateMutations,
  pipeStore: () => pipeStore,
  queryBehaviourSubject: () => queryBehaviourSubject,
  registerStoreExtension: () => registerStoreExtension,
  withStoreUpdates: () => withStoreUpdates
});
module.exports = __toCommonJS(src_exports);

// src/action.ts
var import_rxjs = require("rxjs");
function createAction(operator) {
  const source$ = new import_rxjs.Subject();
  const emitter = (event) => source$.next(event);
  emitter.event$ = operator ? source$.pipe(operator) : source$.asObservable();
  return emitter;
}

// src/effectController.ts
var import_rxjs5 = require("rxjs");

// src/store.ts
var import_rxjs4 = require("rxjs");

// src/compute.ts
var import_rxjs2 = require("rxjs");

// src/utils.ts
var DEFAULT_COMPARATOR = Object.is;
var hasOwnProperty = Object.prototype.hasOwnProperty;
var OBJECT_COMPARATOR = (objA, objB) => {
  if (objA === objB) {
    return true;
  }
  const keysA = Object.keys(objA);
  const keysB = Object.keys(objB);
  if (keysA.length !== keysB.length) {
    return false;
  }
  for (let i = 0; i < keysA.length; i++) {
    const key = keysA[i];
    if (!hasOwnProperty.call(objB, key) || objA[key] !== objB[key]) {
      return false;
    }
  }
  return true;
};
function isReadonlyArray(value) {
  return Array.isArray(value);
}
function removeFromArray(source, item) {
  if (!source)
    return void 0;
  const clone = Array.from(source);
  const index = clone.indexOf(item);
  if (index >= 0) {
    clone.splice(index, 1);
  }
  return clone;
}

// src/compute.ts
var compute = (computation, dependenciesOrOptions) => {
  const options = dependenciesOrOptions ? Array.isArray(dependenciesOrOptions) ? { dependencies: dependenciesOrOptions } : dependenciesOrOptions : void 0;
  const node = createComputationNode(computation, options);
  return createComputationQuery(node);
};
var NODES = /* @__PURE__ */ new WeakMap();
function createComputationNode(computation, options) {
  const dependencies = options == null ? void 0 : options.dependencies;
  return {
    computation,
    comparator: options == null ? void 0 : options.comparator,
    hot: false,
    dependencies: dependencies ? [...new Set(dependencies)] : void 0,
    treeObserverCount: 0
  };
}
function createComputationQuery(node) {
  const query = {
    get: () => getQueryValue(node),
    value$: new import_rxjs2.Observable((observer) => {
      addValueObserver(node, observer);
      return () => removeValueObserver(node, observer);
    })
  };
  NODES.set(query, node);
  return query;
}
var NODE_VERSION = 0;
var STORE_VERSION = 0;
var FAST_QUERY_GETTER = (query, selector) => {
  const value = query.get();
  return selector ? selector(value) : value;
};
function nextVersion(currentValue) {
  return currentValue >= Number.MAX_SAFE_INTEGER ? 0 : currentValue + 1;
}
function nextNodeVersion() {
  STORE_VERSION = nextVersion(STORE_VERSION);
  NODE_VERSION = nextVersion(NODE_VERSION);
}
function nextStoreVersion() {
  STORE_VERSION = nextVersion(STORE_VERSION);
}
function getQueryValue(node) {
  var _a;
  if (((_a = node.valueRef) == null ? void 0 : _a.version) === STORE_VERSION) {
    return node.valueRef.value;
  }
  return node.computation(FAST_QUERY_GETTER);
}
function addValueObserver(node, observer) {
  if (!node.observers) {
    node.observers = [];
  }
  node.observers.push(observer);
  makeHotNode(node, observer);
  updateTreeObserverCount(node);
}
function removeValueObserver(node, observer) {
  node.observers = removeFromArray(node.observers, observer);
  updateTreeObserverCount(node);
}
function getTreeObserverCount(node) {
  var _a;
  const { children, observers } = node;
  let subtreeCount = 0;
  if (children) {
    for (let i = 0; i < children.length; i++) {
      const childNode = children[i];
      subtreeCount += childNode.treeObserverCount;
    }
  }
  return subtreeCount + ((_a = observers == null ? void 0 : observers.length) != null ? _a : 0);
}
function updateTreeObserverCount(node) {
  node.treeObserverCount = getTreeObserverCount(node);
  const parents = node.parents;
  if (parents) {
    for (let i = 0; i < parents.length; i++) {
      const parentNode = parents[i];
      updateTreeObserverCount(parentNode);
    }
  }
  if (node.treeObserverCount === 0) {
    makeColdNode(node);
  }
}
function makeHotNode(node, observer) {
  var _a;
  let dependencies = node.dependencies;
  let valueRef = ((_a = node.valueRef) == null ? void 0 : _a.version) === STORE_VERSION ? node.valueRef : void 0;
  if (dependencies) {
    if (observer && !valueRef) {
      valueRef = node.valueRef = calculate(node.computation);
    }
  } else {
    const visitedDeps = /* @__PURE__ */ new Set();
    const next = calculate(node.computation, (query) => visitedDeps.add(query));
    dependencies = node.dependencies = [...visitedDeps];
    if (observer && !valueRef) {
      valueRef = node.valueRef = next;
    }
  }
  if (dependencies.length > 0 && !node.hot) {
    let depObserver;
    let depsSubscriptions = node.depsSubscriptions;
    if (!depsSubscriptions) {
      depsSubscriptions = node.depsSubscriptions = [];
    }
    for (let i = 0; i < dependencies.length; i++) {
      const parentQuery = dependencies[i];
      if (!parentQuery) {
        throw new TypeError("Incorrect dependency");
      }
      const parentNode = NODES.get(parentQuery);
      if (parentNode) {
        addChildNode(parentNode, node);
      } else {
        if (!depObserver) {
          depObserver = {
            next: () => onSourceChanged(node),
            error: (error) => onSourceError(node, error),
            complete: () => onSourceComplete(node)
          };
        }
        const subscription = parentQuery.value$.subscribe(depObserver);
        depsSubscriptions.push(() => subscription.unsubscribe());
      }
    }
  }
  node.hot = true;
  if (observer && valueRef) {
    observer.next(valueRef.value);
  }
}
function addChildNode(parent, child) {
  if (!parent.children)
    parent.children = [];
  parent.children.push(child);
  if (!child.parents)
    child.parents = [];
  child.parents.push(parent);
  makeHotNode(parent);
}
function makeColdNode(node) {
  node.hot = false;
  node.treeObserverCount = 0;
  const { depsSubscriptions, parents } = node;
  if (depsSubscriptions) {
    for (let i = 0; i < depsSubscriptions.length; i++) {
      const unsubscribe = depsSubscriptions[i];
      unsubscribe();
    }
  }
  if (parents) {
    for (let i = 0; i < parents.length; i++) {
      const parent = parents[i];
      parent.children = removeFromArray(parent.children, node);
    }
  }
  node.valueRef = void 0;
  node.depsSubscriptions = void 0;
  node.observers = void 0;
  node.parents = void 0;
  node.children = void 0;
}
function onSourceChanged(node) {
  nextNodeVersion();
  recompute(node);
}
function onSourceError(node, error) {
  const { children, observers } = node;
  if (observers) {
    for (let i = 0; i < observers.length; i++) {
      const observer = observers[i];
      observer.error(error);
    }
  }
  if (children) {
    for (let i = 0; i < children.length; i++) {
      const child = children[i];
      onSourceError(child, error);
    }
  }
}
function onSourceComplete(node) {
  const { children, observers } = node;
  if (observers) {
    for (let i = 0; i < observers.length; i++) {
      const observer = observers[i];
      observer.complete();
    }
  }
  if (children) {
    for (let i = 0; i < children.length; i++) {
      const child = children[i];
      onSourceComplete(child);
    }
  }
}
function recompute(node) {
  if (node.version === NODE_VERSION) {
    return;
  }
  node.version = NODE_VERSION;
  if (node.observers && node.observers.length > 0) {
    calculateValue(node);
  } else {
    node.valueRef = void 0;
  }
  if (node.treeObserverCount > 0 && node.children) {
    const children = node.children;
    for (let i = 0; i < children.length; i++) {
      const child = children[i];
      recompute(child);
    }
  }
}
function calculateValue(node) {
  var _a;
  const comparator = (_a = node.comparator) != null ? _a : DEFAULT_COMPARATOR;
  const next = calculate(node.computation);
  const isChanged = node.valueRef ? isCalculationChanged(comparator, node.valueRef, next) : true;
  if (isChanged) {
    node.valueRef = next;
    if (node.observers) {
      const observers = node.observers;
      for (let i = 0; i < observers.length; i++) {
        const observer = observers[i];
        observer.next(next.value);
      }
    }
  }
  if (!isChanged && node.valueRef) {
    node.valueRef.version = STORE_VERSION;
  }
}
function calculate(computation, visitor) {
  let params;
  const value = computation(
    (query, selector) => {
      let param = query.get();
      if (selector)
        param = selector(param);
      if (!params)
        params = [];
      params.push(param);
      visitor == null ? void 0 : visitor(query);
      return param;
    }
  );
  return { value, params, version: STORE_VERSION };
}
function isCalculationChanged(comparator, a, b) {
  if (comparator(a.value, b.value)) {
    return false;
  }
  return !(a.params && b.params && isArrayEqual(a.params, b.params));
}
function isArrayEqual(a, b) {
  return a.length === b.length && a.every((value, index) => b[index] === value);
}

// src/storeEvents.ts
var import_rxjs3 = require("rxjs");
var STORE_EVENT_BUS = new import_rxjs3.Subject();

// src/storeMetadata.ts
var MUTATION_NAME_SYMBOL = Symbol();
var INTERNAL_STORE_SYMBOL = Symbol();
function getStateMutationMetadata(mutation) {
  const name = mutation[MUTATION_NAME_SYMBOL];
  return { name };
}
function setStateMutationName(mutation, name) {
  mutation[MUTATION_NAME_SYMBOL] = name;
}
function isInternalStore(store) {
  return store[INTERNAL_STORE_SYMBOL] === true;
}
function setInternalStoreFlag(store) {
  store[INTERNAL_STORE_SYMBOL] = true;
}

// src/store.ts
var STORE_SEQ_NUMBER = 0;
function declareStateUpdates(stateExample, updates) {
  if (updates) {
    return updates;
  }
  return (updates2) => updates2;
}
function pipeStateMutations(mutations) {
  return (state) => mutations.reduce((nextState, mutation) => mutation(nextState), state);
}
var NOTIFY_QUEUE = /* @__PURE__ */ new Set();
function createStore(initialState, options) {
  var _a;
  const stateComparator = (_a = options == null ? void 0 : options.comparator) != null ? _a : DEFAULT_COMPARATOR;
  let currentState = initialState;
  const subscribers = /* @__PURE__ */ new Set();
  const store = {
    id: ++STORE_SEQ_NUMBER,
    name: options == null ? void 0 : options.name,
    value$: new import_rxjs4.Observable((subscriber) => {
      subscribers.add(subscriber);
      subscriber.next(currentState);
      return () => subscribers.delete(subscriber);
    }),
    get() {
      return currentState;
    },
    query(selector) {
      return selector ? compute((get) => selector(get(store)), [store]) : store;
    },
    set(nextState) {
      update(() => nextState);
    },
    update,
    notify() {
      const pinnedState = currentState;
      subscribers.forEach((subscriber) => subscriber.next(pinnedState));
    },
    destroy() {
      var _a2;
      subscribers.forEach((subscriber) => subscriber.complete());
      subscribers.clear();
      STORE_EVENT_BUS.next({ type: "destroyed", store });
      (_a2 = options == null ? void 0 : options.onDestroy) == null ? void 0 : _a2.call(options);
    }
  };
  function update(mutation) {
    const prevState = currentState;
    const nextState = isReadonlyArray(mutation) ? applyMutations(currentState, mutation) : applyMutation(currentState, mutation);
    if (stateComparator(prevState, nextState)) {
      return;
    }
    nextStoreVersion();
    currentState = nextState;
    scheduleNotify(store);
    STORE_EVENT_BUS.next({
      type: "updated",
      store,
      nextState,
      prevState
    });
  }
  function applyMutations(state, mutations) {
    let nextState = state;
    for (let i = 0; i < mutations.length; i++) {
      const mutation = mutations[i];
      if (mutation) {
        nextState = applyMutation(nextState, mutation);
      }
    }
    return nextState;
  }
  function applyMutation(state, mutation) {
    const nextState = mutation(state);
    STORE_EVENT_BUS.next({
      type: "mutation",
      store,
      mutation,
      nextState,
      prevState: state
    });
    return nextState;
  }
  if (options == null ? void 0 : options.internal) {
    setInternalStoreFlag(store);
  }
  STORE_EVENT_BUS.next({ type: "created", store });
  return store;
}
function scheduleNotify(store) {
  if (NOTIFY_QUEUE.add(store).size === 1) {
    Promise.resolve().then(executeNotifyQueue);
  }
}
function executeNotifyQueue() {
  const queue = [...NOTIFY_QUEUE];
  NOTIFY_QUEUE.clear();
  queue.forEach((store) => store.notify());
}
function createStoreUpdates(storeUpdate, stateUpdates) {
  const updates = {};
  Object.entries(stateUpdates).forEach(([key, mutationFactory]) => {
    updates[key] = (...args) => {
      const mutation = mutationFactory(...args);
      setStateMutationName(mutation, key);
      storeUpdate(mutation);
    };
  });
  return updates;
}
function withStoreUpdates(store, updates) {
  const storeUpdates = createStoreUpdates(store.update, updates);
  return { ...store, updates: storeUpdates };
}

// src/effectController.ts
var GLOBAL_EFFECT_UNHANDLED_ERROR_SUBJECT = new import_rxjs5.Subject();
var GLOBAL_EFFECT_UNHANDLED_ERROR$ = GLOBAL_EFFECT_UNHANDLED_ERROR_SUBJECT.asObservable();
function emitGlobalUnhandledError(effectError) {
  if (GLOBAL_EFFECT_UNHANDLED_ERROR_SUBJECT.observed) {
    GLOBAL_EFFECT_UNHANDLED_ERROR_SUBJECT.next(effectError);
  } else {
    console.error("Uncaught error in Effect", effectError);
  }
}
var increaseCount = (count) => count + 1;
var decreaseCount = (count) => count > 0 ? count - 1 : 0;
function createEffectController() {
  const subscriptions = new import_rxjs5.Subscription();
  const event$ = new import_rxjs5.Subject();
  const done$ = new import_rxjs5.Subject();
  const error$ = new import_rxjs5.Subject();
  const pendingCount = createStore(0, {
    internal: true
  });
  subscriptions.add(() => {
    event$.complete();
    done$.complete();
    error$.complete();
    pendingCount.destroy();
  });
  const notifications$ = (0, import_rxjs5.merge)(
    done$.pipe(
      (0, import_rxjs5.map)((entry) => ({ type: "result", ...entry }))
    ),
    error$.pipe(
      (0, import_rxjs5.map)((entry) => ({ type: "error", ...entry }))
    )
  );
  return {
    state: {
      done$: done$.asObservable(),
      result$: done$.pipe((0, import_rxjs5.map)(({ result }) => result)),
      error$: error$.asObservable(),
      final$: notifications$,
      pending: pendingCount.query((count) => count > 0),
      pendingCount: pendingCount.query(import_rxjs5.identity)
    },
    start: () => pendingCount.update(increaseCount),
    next: (result) => done$.next(result),
    complete: () => pendingCount.update(decreaseCount),
    error: (effectError) => {
      if (effectError.origin === "handler") {
        pendingCount.update(decreaseCount);
      }
      if (error$.observed) {
        error$.next(effectError);
      } else {
        emitGlobalUnhandledError(effectError);
      }
    },
    destroy: () => {
      subscriptions.unsubscribe();
    }
  };
}

// src/effect.ts
var import_rxjs6 = require("rxjs");
var DEFAULT_MERGE_MAP_PIPELINE = (eventProject) => (0, import_rxjs6.mergeMap)(eventProject);
function createEffect(handler, options) {
  var _a;
  const pipeline = (_a = options == null ? void 0 : options.pipeline) != null ? _a : DEFAULT_MERGE_MAP_PIPELINE;
  const event$ = new import_rxjs6.Subject();
  const controller = createEffectController();
  const subscriptions = new import_rxjs6.Subscription(() => {
    event$.complete();
    controller.destroy();
  });
  const eventProject = (event) => {
    return (0, import_rxjs6.defer)(() => {
      controller.start();
      const result = handler(event);
      return result instanceof import_rxjs6.Observable || result instanceof Promise ? result : (0, import_rxjs6.of)(result);
    }).pipe(
      (0, import_rxjs6.tap)({
        next: (result) => {
          controller.next({ event, result });
        },
        complete: () => {
          controller.complete();
        },
        error: (error) => {
          controller.error({ origin: "handler", event, error });
        }
      })
    );
  };
  subscriptions.add(event$.pipe(pipeline(eventProject), (0, import_rxjs6.retry)()).subscribe());
  return {
    ...controller.state,
    handle(source) {
      const observable = getSourceObservable(source);
      const subscription = observable.subscribe({
        next: (event) => event$.next(event),
        error: (error) => controller.error({ origin: "source", error })
      });
      subscriptions.add(subscription);
      return subscription;
    },
    destroy: () => {
      subscriptions.unsubscribe();
    }
  };
}
function getSourceObservable(source) {
  const type = typeof source;
  if (type === "function" && "event$" in source) {
    return source.event$;
  }
  if (type === "object" && "value$" in source) {
    return source.value$;
  }
  if (source instanceof import_rxjs6.Observable) {
    return source;
  }
  throw new TypeError("Unexpected source type");
}

// src/scope.ts
var import_rxjs7 = require("rxjs");
function createScope() {
  const subscriptions = new import_rxjs7.Subscription();
  function registerTeardown(teardown) {
    subscriptions.add(teardown);
  }
  function destroy() {
    subscriptions.unsubscribe();
  }
  function createController2(factory) {
    const controller = factory();
    registerTeardown(controller.destroy);
    return controller;
  }
  return {
    add: registerTeardown,
    onDestroy: registerTeardown,
    destroy,
    createController: createController2,
    createStore(initialState, options) {
      return createController2(() => createStore(initialState, options));
    },
    createEffect(handler, options) {
      return createController2(
        () => createEffect(handler, options)
      );
    },
    handle(source, handler, options) {
      const effect = createController2(
        () => createEffect(handler, options)
      );
      effect.handle(source);
      return effect;
    },
    subscribe(source, nextOrObserver) {
      const subscription = source.subscribe(nextOrObserver);
      registerTeardown(subscription);
      return subscription;
    }
  };
}

// src/mvc.ts
var import_ditox = require("ditox");
function createController(factory) {
  const scope = createScope();
  const controller = factory(scope);
  return {
    ...controller,
    destroy: () => {
      var _a;
      (_a = controller.destroy) == null ? void 0 : _a.call(controller);
      scope.destroy();
    }
  };
}
function declareController(tokens, factory) {
  return (0, import_ditox.injectable)(
    (deps) => createController((scope) => factory(deps, scope)),
    tokens
  );
}
function declareViewController(tokensOrFactory, factory) {
  return (container, ...params) => {
    if (typeof tokensOrFactory === "function") {
      return createController((scope) => {
        return tokensOrFactory(scope, ...params);
      });
    }
    return (0, import_ditox.injectable)((dependencies) => {
      return createController((scope) => {
        const factoryValue = factory;
        const result = factoryValue(dependencies, scope);
        if (typeof result === "function") {
          return result(scope, ...params);
        }
        return result;
      });
    }, tokensOrFactory)(container);
  };
}

// src/queryMappers.ts
function mapQuery(query, mapper) {
  return compute((get) => mapper(get(query)), [query]);
}
function mergeQueries(queries, merger) {
  return compute((get) => {
    const values = queries.map((query) => get(query));
    return merger(...values);
  }, queries);
}

// src/queryUtils.ts
function queryBehaviourSubject(source) {
  return { get: () => source.value, value$: source };
}

// src/pipeStore.ts
function pipeStore(store, operator) {
  let subscription;
  const clone = createStore(store.get(), {
    internal: true,
    onDestroy: () => {
      if (subscription) {
        subscription.unsubscribe();
        subscription = void 0;
      }
    }
  });
  subscription = store.value$.pipe(operator).subscribe({
    next: (state) => {
      clone.set(state);
    },
    complete: () => {
      clone.destroy();
    }
  });
  return clone;
}

// src/storeExtensions.ts
function registerStoreExtension(extension) {
  const api = {
    getStateMutationMetadata
  };
  const middleware = extension(api);
  return STORE_EVENT_BUS.subscribe((event) => {
    if (middleware.onStoreEvent && !isInternalStore(event.store)) {
      middleware.onStoreEvent(event);
    }
  });
}

// src/storeLoggerExtension.ts
function createStoreLoggerExtension(logger) {
  return (api) => ({
    onStoreEvent(event) {
      var _a, _b;
      const { type, store } = event;
      const storeName = `${(_a = store.name) != null ? _a : ""}#${store.id}`;
      switch (type) {
        case "created":
        case "destroyed": {
          logger(storeName, type);
          break;
        }
        case "updated": {
          logger(storeName, type, {
            nextState: event.nextState,
            prevState: event.prevState
          });
          break;
        }
        case "mutation": {
          const mutationName = (_b = api.getStateMutationMetadata(event.mutation).name) != null ? _b : "anonymous";
          logger(storeName, type, mutationName, {
            nextState: event.nextState,
            prevState: event.prevState
          });
        }
      }
    }
  });
}

// src/declareStore.ts
function declareStore(declaration) {
  const {
    initialState: baseState,
    updates,
    options: baseOptions
  } = declaration;
  function factory(initialState, options) {
    const state = initialState === void 0 ? baseState : typeof initialState === "function" ? initialState(baseState) : initialState;
    const store = createStore(state, {
      ...baseOptions,
      ...options
    });
    return withStoreUpdates(store, updates);
  }
  return Object.assign(factory, { updates });
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  GLOBAL_EFFECT_UNHANDLED_ERROR$,
  OBJECT_COMPARATOR,
  compute,
  createAction,
  createController,
  createEffect,
  createEffectController,
  createScope,
  createStore,
  createStoreLoggerExtension,
  createStoreUpdates,
  declareController,
  declareStateUpdates,
  declareStore,
  declareViewController,
  mapQuery,
  mergeQueries,
  pipeStateMutations,
  pipeStore,
  queryBehaviourSubject,
  registerStoreExtension,
  withStoreUpdates
});
